/*
 * Copyright (c) 2004-2006 Weswit s.r.l., Via Campanini, 6 - 20124 Milano, Italy.
 * All rights reserved.
 * www.lightstreamer.com
 *
 * This software is the confidential and proprietary information of
 * Weswit s.r.l.
 * You shall not disclose such Confidential Information and shall use it
 * only in accordance with the terms of the license agreement you entered
 * into with Weswit s.r.l.
 */


package com.lightstreamer.adapters.stocklist_demo;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.lightstreamer.interfaces.data.DataProvider;
import com.lightstreamer.interfaces.data.IndexedItemEvent;
import com.lightstreamer.interfaces.data.ItemEvent;
import com.lightstreamer.interfaces.data.ItemEventListener;
import com.lightstreamer.interfaces.data.SubscriptionException;

/**
 * This Data Adapter accepts a limited set of item names (the names starting
 * with "item") and listens to a (simulated) stock quotes feed, waiting for
 * update events. The events pertaining to the currently subscribed items
 * are then forwarded to Lightstreamer.
 * This example demonstrates how a Data Adapter could interoperate with
 * a broadcast feed, which sends data for all items. Many other types of feeds
 * may exist, with very different behaviours.
 */
public class DemoDataAdapter implements DataProvider {

    private ItemEventListener listener;
    private final HashMap subscribedItems = new HashMap();
    private final ExternalFeedSimulator myFeed;

    /*
     * used for the OrderedItemEvent version only (currently commented out)
     */
    private static final String[] names = new String[]{"stock_name", "time", 
                                                       "last_price",
                                                       "ask", "bid",
                                                       "bid_quantity",
                                                       "ask_quantity",
                                                       "pct_change", "min",
                                                       "max", "ref_price",
                                                       "open_price"};
    private static final HashMap codes = new HashMap() {
        {
            for (int code = 0; code < names.length; code++) {
                put(names[code], new Integer(code));
            }
        }
    };

    public DemoDataAdapter() {
        myFeed = new ExternalFeedSimulator();
    }

    public void init(Map params, File configDir) {
        myFeed.start();
    }

    public void setListener(ItemEventListener listener) {
        this.listener = listener;
        myFeed.setFeedListener(new MyFeedListener());
    }

    public void subscribe(String itemName, boolean needsIterator)
            throws SubscriptionException {
        if (needsIterator) {
            // OK, as we will always send HashMap objects
        }
        if (itemName.startsWith("item")) {
            synchronized (subscribedItems) {
                subscribedItems.put(itemName, new Boolean(false));
            }
            // now we ask the feed for the snapshot; our feed will insert
            // an event with snapshot information into the normal updates flow
            myFeed.sendCurrentValues(itemName);
        } else {
            throw new SubscriptionException("Unexpected item: " + itemName);
        }
    }

    public void unsubscribe(String itemName) {
        synchronized (subscribedItems) {
            subscribedItems.remove(itemName);
        }
    }

    public boolean isSnapshotAvailable(String itemName) {
        return true;
    }

    private class MyFeedListener implements ExternalFeedListener {

        /**
         * Called by our feed for each update event occurrence on some stock.
         * If isSnapshot is true, then the event contains a full snapshot
         * with the current values of all fields for the stock.
         */
        public void onEvent(String itemName, final HashMap currentValues,
                            boolean isSnapshot) {
            synchronized (subscribedItems) {
                if (!subscribedItems.containsKey(itemName)) {
                    return;
                }
                Boolean started = (Boolean) subscribedItems.get(itemName);
                boolean snapshotReceived = started.booleanValue();
                if (!snapshotReceived) {
                    if (!isSnapshot) {
                        // we ignore the update and keep waiting until
                        // a full snapshot for the item has been received
                        return;
                    }
                    subscribedItems.put(itemName, new Boolean(true));
                } else {
                    if (isSnapshot) {
                        // it's not the first event we have received carrying
                        // snapshot information for the item; so, this event
                        // is not a snapshot from Lightstreamer point of view
                        isSnapshot = false;
                    }
                }

                // We should ensure that update cannot be called after
                // unsubscribe, so we need to hold the subscribedItems lock;
                // however, update is nonblocking; moreover, it only takes locks
                // to first order mutexes; so, it can safely be called here

                // Note that, in case a rapid subscribe-unsubscribe-subscribe
                // sequence has just been issued for this item,
                // we may still be receiving and forwarding the snapshot
                // related with the first subscribe call;
                // this case still leads to a perfectly consistent update flow,
                // in this scenario, so no checks are inserted to detect the case

                // For better control of the update flow, the SmartDataProvider
                // interface can be implemented instead of the simpler DataProvider

                /* basic version:
                 */
                listener.update(itemName, currentValues, isSnapshot);
    
                /* ItemEvent version:
                listener.update(itemName, new ItemEvent() {
    
                    public Iterator getNames() {
                        return currentValues.keySet().iterator();
                    }
    
                    public Object getValue(String name) {
                        return currentValues.get(name);
                    }
    
                }, isSnapshot);
                */
    
                /* OrderedItemEvent version:
                listener.update(itemName, new IndexedItemEvent() {
    
                    public int getMaximumIndex() {
                        return names.length - 1;
                    }
    
                    public int getIndex(String name) {
                        Object keyObject = codes.get(name);
                        if (keyObject == null) {
                            return -1;
                        } else {
                            return ((Integer) keyObject).intValue();
                        }
                    }
    
                    public String getName(int index) {
                        if (currentValues.get(names[index]) == null) {
                            return null;
                        } else {
                            return names[index];
                        }
                    }
    
                    public Object getValue(int index) {
                        return currentValues.get(names[index]);
                    }
    
                }, isSnapshot);
                */
            }

        }

    }

}


/*--- Formatted in Lightstreamer Java Convention Style on 2005-11-03 ---*/
